/*
 * Copyright (C) 1996-1997 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#pragma ident	"@(#)lxruninit.c	1.5	99/02/24 SMI"

#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/socket.h>
#include	<netdb.h>
#include	"lxrun.h"
#include	"devmap.h"

char	*linux_root	= NULL;
char	*linux_uname	= NULL;

int parse_pathmap_table(char *pathmapfn);

/*
** Perform initializations common to lxrun.c and ld-linux.c
*/

void
lxrun_init(void)
{
	char		*display;
#ifdef PATHMAPFILE
	char		*pathmapfn;
	char		*pathmapdef;
#endif

	/* Must be static to make putenv() happy. */
	static char	displaybuf[64];

#ifdef DEF_LINUX_ROOT
	if ((linux_root = getenv("LINUX_ROOT")) == NULL)
		linux_root = DEF_LINUX_ROOT;
#else
	linux_root = getenv("LINUX_ROOT");
#endif

	linux_uname = getenv("LINUX_UNAME");

#ifndef NO_DISPLAY_HACK
	/*
	** For local displays, OpenServer uses ":0.0" and UnixWare uses
	** "unix:0.0".  The former causes Linux X11 binaries to try to
	** use shared memory, which won't work with the native X server.
	** The latter causes a name lookup for the machine "unix" which
	** probably fails.  We'll detect these two cases and set DISPLAY
	** to contain a valid machine name instead.
	** Note that this causes a loss in performance when running X apps
	** since we have to go through TCP.
	*/
	if ((display = getenv("DISPLAY")) != NULL &&
#if defined(UNIXWARE) || defined(GEMINI)
		/* These platforms use "unix:0.0" for local X */
		(*((long *)display)==*((long *)"unix") && display[4]==':')
#else
		/* Other platforms use ":0.0" */
		(display[0] == ':')
#endif
			) {

#if defined(UNIXWARE) || defined(GEMINI)
		display+=4;		/* skip "unix" */
#endif

		strcpy(displaybuf, "DISPLAY=127.0.0.1");
		strcat(displaybuf, display);
		
		if (putenv(displaybuf)) {
			fprintf(stderr, "putenv(%s) failed\n", displaybuf);
			exit(1);
		}
	}
#endif

	if (init_signals() != 0)
		fatal(errno, "can't set up signal handlers");

	devmap_init();

#ifdef PATHMAPFILE

	pathmapdef=NULL;
	if(!(pathmapfn = getenv("LINUX_PATHMAP")))
		{
		/* Assume $LINUX_ROOT/PATHMAP */
#if	defined(SOLARIS)
		/* XXX this is a bit wrong */
		pathmapdef = strdup("/usr/local/lxrun/PATHMAP");
#else
		pathmapdef = (char *)malloc(strlen(linux_root)+9);
		strcpy(pathmapdef, linux_root);
		strcat(pathmapdef, "/PATHMAP");
#endif
		pathmapfn = pathmapdef;
		}
	if(parse_pathmap_table(pathmapfn) < 0)
		fatal(errno, "Can't open pathmap file \"%s\"",
					pathmapfn);
	if(pathmapdef)
		free(pathmapdef);

#endif	/* PATHMAPFILE */
}
